@@ -18,6 +18,7 @@ from .types import (
     SomeShape,
     SomeResolution,
     SomeIndex2d,
+    Unset,
     xy_,
     yx_,
     res_,
@@ -60,6 +61,7 @@ __all__ = [
     "Geometry",
     "MaybeCRS",
     "SomeCRS",
+    "Unset",
     "xy_",
     "yx_",
     "res_",
