@@ -32,9 +32,9 @@ from .crs import CRS, CRSError, SomeCRS, norm_crs_or_error
 from .gcp import GCPGeoBox, GCPMapping
 from .geobox import Coordinate, GeoBox
 from .geom import Geometry
-from .math import affine_from_axis
+from .math import affine_from_axis, maybe_int, resolution_from_affine
 from .overlap import compute_output_geobox
-from .types import Resolution, resxy_, xy_
+from .types import Resolution, xy_
 
 # pylint: disable=import-outside-toplevel
 if have.rasterio:
@@ -50,6 +50,9 @@ SomeGeoBox = Union[GeoBox, GCPGeoBox]
 
 _DEFAULT_CRS_COORD_NAME = "spatial_ref"
 
+# these attributes are pruned during reproject
+SPATIAL_ATTRIBUTES = ("crs", "crs_wkt", "grid_mapping", "gcps", "epsg")
+
 
 @dataclass
 class GeoState:
@@ -58,6 +61,7 @@ class GeoState:
     """
 
     spatial_dims: Optional[Tuple[str, str]] = None
+    crs_coord: Optional[xarray.DataArray] = None
     transform: Optional[Affine] = None
     crs: Optional[CRS] = None
     geobox: Optional[SomeGeoBox] = None
@@ -154,6 +158,7 @@ def _mk_crs_coord(
     crs: CRS,
     name: str = _DEFAULT_CRS_COORD_NAME,
     gcps=None,
+    transform: Optional[Affine] = None,
 ) -> xarray.DataArray:
     # pylint: disable=protected-access
 
@@ -164,6 +169,9 @@ def _mk_crs_coord(
     if gcps is not None:
         cf["gcps"] = _gcps_to_json(gcps)
 
+    if transform is not None:
+        cf["GeoTransform"] = " ".join(map(str, transform.to_gdal()))
+
     return xarray.DataArray(
         numpy.asarray(epsg, "int32"),
         name=name,
@@ -247,8 +255,8 @@ def xr_coords(
        CRS coordinate at all.
 
     :returns:
-      Dictionary ``name:str -> xr.DataArray``. Where names are either ``y,x`` for projected or
-      ``latitude, longitude`` for geographic.
+       Dictionary ``name:str -> xr.DataArray``. Where names are either ``y,x`` for projected or
+       ``latitude, longitude`` for geographic.
 
     """
     attrs = {}
@@ -257,6 +265,7 @@ def xr_coords(
         attrs["crs"] = str(crs)
 
     gcps = None
+    transform: Optional[Affine] = None
 
     if isinstance(gbox, GCPGeoBox):
         coords: Dict[Hashable, xarray.DataArray] = {
@@ -264,19 +273,23 @@ def xr_coords(
             for name, sz in zip(gbox.dimensions, gbox.shape)
         }
         gcps = gbox.gcps()
-    elif gbox.axis_aligned:
-        coords = {
-            name: _coord_to_xr(name, coord, **attrs)
-            for name, coord in gbox.coordinates.items()
-        }
     else:
-        coords = {
-            name: _mk_pixel_coord(name, sz, gbox.transform)
-            for name, sz in zip(gbox.dimensions, gbox.shape)
-        }
+        transform = gbox.transform
+        if gbox.axis_aligned:
+            coords = {
+                name: _coord_to_xr(name, coord, **attrs)
+                for name, coord in gbox.coordinates.items()
+            }
+        else:
+            coords = {
+                name: _mk_pixel_coord(name, sz, transform)
+                for name, sz in zip(gbox.dimensions, gbox.shape)
+            }
 
     if crs_coord_name is not None and crs is not None:
-        coords[crs_coord_name] = _mk_crs_coord(crs, crs_coord_name, gcps=gcps)
+        coords[crs_coord_name] = _mk_crs_coord(
+            crs, crs_coord_name, gcps=gcps, transform=transform
+        )
 
     return coords
 
@@ -347,26 +360,53 @@ def _extract_gcps(crs_coord: xarray.DataArray) -> Optional[GCPMapping]:
         return None
 
 
-def _extract_transform(src: XarrayObject, sdims: Tuple[str, str]) -> Optional[Affine]:
+def _extract_geo_transform(crs_coord: xarray.DataArray) -> Optional[Affine]:
+    geo_transfrom_parts = crs_coord.attrs.get("GeoTransform", "").split(" ")
+    if len(geo_transfrom_parts) != 6:
+        return None
+    try:
+        c, a, b, f, d, e = map(float, geo_transfrom_parts)
+    except ValueError:
+        return None
+
+    return Affine.from_gdal(c, a, b, f, d, e)
+
+
+def _extract_transform(
+    src: XarrayObject,
+    sdims: Tuple[str, str],
+    crs_coord: Optional[xarray.DataArray],
+    gcp: bool,
+) -> Optional[Affine]:
     if any(dim not in src.coords for dim in sdims):
         # special case of no spatial dims at all
-        # happens for GCP sources loaded by rioxarray
-        return None
+        # happens for GCP/rotated sources loaded by rioxarray
+        if gcp or crs_coord is None:
+            return None
+        return _extract_geo_transform(crs_coord)
 
     _yy, _xx = (src[dim] for dim in sdims)
-    rx, ry = (coord.attrs.get("resolution", None) for coord in (_xx, _yy))
-    fallback_res = None
-    if rx is not None and ry is not None:
-        fallback_res = resxy_(float(rx), float(ry))
 
+    # First try to compute from 1-D X/Y coords
     try:
-        transform = affine_from_axis(_xx.values, _yy.values, fallback_res)
+        transform = affine_from_axis(_xx.values, _yy.values)
     except ValueError:
-        # this can fail when any dimension is shorter than 2 elements
-        return None
+        # This can fail when any dimension is shorter than 2 elements
+        # Figure out fallback resolution if possible and try again
+        if crs_coord is None:
+            return None
+        if (original_transform := _extract_geo_transform(crs_coord)) is None:
+            return None
+        try:
+            transform = affine_from_axis(
+                _xx.values,
+                _yy.values,
+                resolution_from_affine(original_transform),
+            )
+        except ValueError:
+            return None
 
-    _pix2world = _xx.encoding.get("_transform", None)
-    if _pix2world is not None:
+    if not gcp and (_pix2world := _xx.encoding.get("_transform", None)) is not None:
         # non-axis aligned geobox detected
         # adjust transform
         #  world <- pix' <- pix
@@ -380,7 +420,7 @@ def _locate_geo_info(src: XarrayObject) -> GeoState:
     if sdims is None:
         return GeoState()
 
-    transform = _extract_transform(src, sdims)
+    crs_coord: Optional[xarray.DataArray] = None
     crs: Optional[CRS] = None
     geobox: Optional[SomeGeoBox] = None
     gcp: Optional[GCPMapping] = None
@@ -392,19 +432,27 @@ def _locate_geo_info(src: XarrayObject) -> GeoState:
     if num_candidates > 0:
         if num_candidates > 1:
             warnings.warn("Multiple CRS coordinates are present")
-        crs = _extract_crs(_crs_coords[0])
-        gcp = _extract_gcps(_crs_coords[0])
+        crs_coord = _crs_coords[0]
+        crs = _extract_crs(crs_coord)
+        gcp = _extract_gcps(crs_coord)
     else:
         # try looking in attributes
         crs = _get_crs_from_attrs(src, sdims)
 
+    transform = _extract_transform(src, sdims, crs_coord, gcp is not None)
+
     if gcp is not None:
         geobox = GCPGeoBox((ny, nx), gcp, transform)
     elif transform is not None:
         geobox = GeoBox((ny, nx), transform, crs)
 
     return GeoState(
-        spatial_dims=sdims, transform=transform, crs=crs, geobox=geobox, gcp=gcp
+        spatial_dims=sdims,
+        crs_coord=crs_coord,
+        transform=transform,
+        crs=crs,
+        geobox=geobox,
+        gcp=gcp,
     )
 
 
@@ -489,6 +537,7 @@ def _xr_reproject_da(
 
     This method uses :py:mod:`rasterio`.
     """
+    # pylint: disable=too-many-locals
     assert isinstance(src, xarray.DataArray)
 
     if have.rasterio is False:  # pragma: nocover
@@ -497,55 +546,74 @@ def _xr_reproject_da(
     assert isinstance(src.odc, ODCExtensionDa)  # for mypy sake
     src_gbox = src.odc.geobox
 
-    if src_gbox is None:
+    if src_gbox is None or src_gbox.crs is None:
         raise ValueError("Can not reproject non-georegistered array.")
 
+    resolution = kw.pop("resolution", "auto")
+    tight = kw.pop("tight", False)
+
     if isinstance(how, GeoBox):
         dst_geobox = how
     else:
-        dst_geobox = src.odc.output_geobox(how)
-
-    if is_dask_collection(src):
-        raise NotImplementedError("Dask inputs are not yet supported.")
+        dst_geobox = src.odc.output_geobox(how, resolution=resolution, tight=tight)
 
     # compute destination shape by replacing spatial dimensions shape
     ydim = src.odc.ydim
     assert ydim + 1 == src.odc.xdim
     dst_shape = (*src.shape[:ydim], *dst_geobox.shape, *src.shape[ydim + 2 :])
 
-    dst = numpy.empty(dst_shape, dtype=src.dtype)
     src_nodata = kw.pop("src_nodata", None)
     if src_nodata is None:
         src_nodata = src.odc.nodata
     if dst_nodata is None:
         dst_nodata = src_nodata
 
-    dst = rio_reproject(
-        src.values,
-        dst,
-        src_gbox,
-        dst_geobox,
-        resampling=resampling,
-        src_nodata=src_nodata,
-        dst_nodata=dst_nodata,
-        ydim=ydim,
-        **kw,
-    )
+    if is_dask_collection(src):
+        from ._dask import _dask_rio_reproject
 
-    attrs = src.attrs.copy()
+        dst: Any = _dask_rio_reproject(
+            src.data,
+            src_gbox,
+            dst_geobox,
+            resampling=resampling,
+            src_nodata=src_nodata,
+            dst_nodata=dst_nodata,
+            ydim=ydim,
+            **kw,
+        )
+    else:
+        dst = numpy.empty(dst_shape, dtype=src.dtype)
+
+        dst = rio_reproject(
+            src.values,
+            dst,
+            src_gbox,
+            dst_geobox,
+            resampling=resampling,
+            src_nodata=src_nodata,
+            dst_nodata=dst_nodata,
+            ydim=ydim,
+            **kw,
+        )
+
+    attrs = {k: v for k, v in src.attrs.items() if k not in SPATIAL_ATTRIBUTES}
     if dst_nodata is None:
         attrs.pop("nodata", None)
         attrs.pop("_FillValue", None)
     else:
-        attrs.update(nodata=dst_nodata)
+        attrs.update(nodata=maybe_int(dst_nodata, 1e-6))
 
     # new set of coords (replace x,y dims)
-    coords = dict((k, v) for k, v in src.coords.items() if k not in src.dims)
+    sdims = src.odc.spatial_dims
+    assert sdims is not None
+    coords = dict((k, v) for k, v in src.coords.items() if k not in sdims)
     coords.update(xr_coords(dst_geobox))
 
     dims = (*src.dims[:ydim], *dst_geobox.dimensions, *src.dims[ydim + 2 :])
 
-    return xarray.DataArray(dst, coords=coords, dims=dims, attrs=attrs)
+    out = xarray.DataArray(dst, coords=coords, dims=dims, attrs=attrs)
+    out.encoding["grid_mapping"] = _DEFAULT_CRS_COORD_NAME
+    return out
 
 
 class ODCExtension:
@@ -715,6 +783,7 @@ def wrap_xr(
     time=None,
     nodata=None,
     crs_coord_name: Optional[str] = _DEFAULT_CRS_COORD_NAME,
+    axis: Optional[int] = None,
     **attrs,
 ) -> xarray.DataArray:
     """
@@ -727,11 +796,20 @@ def wrap_xr(
     :param attrs: Any other attributes to set on the result
     :return: xarray DataArray
     """
-    assert im.shape[-2:] == gbox.shape
+    if axis is None:
+        axis = 1 if time is not None else 0
 
-    prefix_dims: Tuple[str, ...] = ("time",) if im.ndim > 2 else ()
+    if im.ndim == 2 and axis == 1:
+        im = im[numpy.newaxis, ...]
 
-    dims = (*prefix_dims, *gbox.dimensions)
+    assert axis in (0, 1)  # upto 1 extra dimension on the left only
+    assert im.ndim - axis - 2 in (0, 1)  # upto 1 extra dimension on the right only
+    assert im.shape[axis : axis + 2] == gbox.shape
+
+    prefix_dims: Tuple[str, ...] = ("time",) if axis == 1 else ()
+    postfix_dims: Tuple[str, ...] = ("band",) if im.ndim - axis > 2 else ()
+
+    dims = (*prefix_dims, *gbox.dimensions, *postfix_dims)
     coords = xr_coords(gbox, crs_coord_name=crs_coord_name)
 
     if time is not None:
@@ -742,6 +820,10 @@ def wrap_xr(
             time = xarray.DataArray(time, dims=prefix_dims).astype("datetime64[ns]")
 
         coords["time"] = time
+    if postfix_dims:
+        coords["band"] = xarray.DataArray(
+            [f"b{i}" for i in range(im.shape[-1])], dims=postfix_dims
+        )
 
     if nodata is not None:
         attrs = dict(nodata=nodata, **attrs)
