@@ -3,7 +3,19 @@
 # Copyright (c) 2015-2020 ODC Contributors
 # SPDX-License-Identifier: Apache-2.0
 import warnings
-from typing import TYPE_CHECKING, Any, Callable, Dict, Optional, Tuple, Union, overload
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    Hashable,
+    List,
+    Optional,
+    Protocol,
+    Tuple,
+    Union,
+    overload,
+)
 
 import cachetools
 import numpy
@@ -12,30 +24,58 @@ from pyproj.enums import WktVersion
 from pyproj.exceptions import CRSError
 from pyproj.transformer import Transformer
 
-SomeCRS = Union[str, "CRS", _CRS, Dict[str, Any]]
-MaybeCRS = Optional[SomeCRS]
+from .types import XY, Unset
 
-_crs_cache: Dict[str, Tuple[_CRS, str, Optional[int]]] = {}
+SomeCRS = Union[str, int, "CRS", _CRS, Dict[str, Any]]
+MaybeCRS = Union[SomeCRS, Unset, None]
+EPSG_UNSET = 0
 
 
-def _make_crs_key(crs_spec: Union[str, _CRS]) -> str:
+class CRSLike(Protocol):
+    """CRS Like object."""
+
+    def to_wkt(self, *args, **kw) -> str:
+        ...
+
+
+_crs_cache: Dict[Hashable, Tuple[_CRS, str, Optional[int]]] = {}
+
+
+def _make_crs_key(crs_spec: Union[int, str, Hashable, CRSLike]) -> Hashable:
     if isinstance(crs_spec, str):
         normed_epsg = crs_spec.upper()
         if normed_epsg.startswith("EPSG:"):
             return normed_epsg
         return crs_spec
+    if isinstance(crs_spec, int):
+        return f"EPSG:{crs_spec}"
+    if isinstance(crs_spec, Hashable):
+        return crs_spec
+
     return crs_spec.to_wkt()
 
 
 @cachetools.cached(_crs_cache, key=_make_crs_key)
-def _make_crs(crs: Union[str, _CRS]) -> Tuple[_CRS, str, Optional[int]]:
-    if isinstance(crs, str):
-        crs = _CRS.from_user_input(crs)
-    epsg = crs.to_epsg()
-    if epsg is not None:
-        crs_str = f"EPSG:{epsg}"
+def _make_crs(
+    crs_spec: Union[str, int, _CRS, CRSLike]
+) -> Tuple[_CRS, str, Optional[int]]:
+    epsg = EPSG_UNSET
+    if isinstance(crs_spec, str):
+        crs = _CRS.from_user_input(crs_spec)
+    elif isinstance(crs_spec, int):
+        epsg = crs_spec
+        crs = _CRS.from_epsg(crs_spec)
+    elif isinstance(crs_spec, _CRS):
+        crs = crs_spec
     else:
-        crs_str = crs.to_wkt()
+        crs = _CRS.from_wkt(crs_spec.to_wkt())
+
+    crs_str = str(crs)
+    crs_str_u = crs_str.upper()
+    if crs_str_u.startswith("EPSG:"):
+        crs_str = crs_str_u
+        epsg = int(crs_str.split(":", 1)[1])
+
     return (crs, crs_str, epsg)
 
 
@@ -44,8 +84,8 @@ def _make_crs_transform_key(from_crs, to_crs, always_xy):
 
 
 @cachetools.cached({}, key=_make_crs_transform_key)
-def _make_crs_transform(from_crs, to_crs, always_xy):
-    return Transformer.from_crs(from_crs, to_crs, always_xy=always_xy).transform
+def _make_crs_transform(from_crs: _CRS, to_crs: _CRS, always_xy: bool) -> Transformer:
+    return Transformer.from_crs(from_crs, to_crs, always_xy=always_xy)
 
 
 class CRS:
@@ -69,35 +109,18 @@ class CRS:
         :raises: :py:class:`pyproj.exceptions.CRSError`
         """
 
-        if isinstance(crs_spec, str):
+        if isinstance(crs_spec, (str, int, _CRS)):
             self._crs, self._str, self._epsg = _make_crs(crs_spec)
         elif isinstance(crs_spec, CRS):
             self._crs = crs_spec._crs
             self._epsg = crs_spec._epsg
             self._str = crs_spec._str
-        elif isinstance(crs_spec, _CRS):
-            self._crs, self._str, self._epsg = _make_crs(crs_spec)
         elif isinstance(crs_spec, dict):
             self._crs, self._str, self._epsg = _make_crs(_CRS.from_dict(crs_spec))
+        elif hasattr(crs_spec, "to_wkt"):
+            self._crs, self._str, self._epsg = _make_crs(crs_spec)
         else:
-            try:
-                epsg = crs_spec.to_epsg()
-            except AttributeError:
-                epsg = None
-            if epsg is not None:
-                self._crs, self._str, self._epsg = _make_crs(f"EPSG:{epsg}")
-                return
-            try:
-                wkt = crs_spec.to_wkt()
-            except AttributeError:
-                wkt = None
-            if wkt is not None:
-                self._crs, self._str, self._epsg = _make_crs(wkt)
-                return
-
-            raise CRSError(
-                "Expect string or any object with `.to_epsg()` or `.to_wkt()` methods"
-            )
+            raise CRSError(f"Unexpected input encountered: {crs_spec}")
 
     def __getstate__(self):
         return {"crs_str": self._str}
@@ -126,6 +149,8 @@ class CRS:
         """
         EPSG Code of the CRS or ``None``.
         """
+        if self._epsg == EPSG_UNSET:
+            self._epsg = self._crs.to_epsg()
         return self._epsg
 
     @property
@@ -133,7 +158,7 @@ class CRS:
         """
         EPSG Code of the CRS or ``None``.
         """
-        return self._epsg
+        return self.to_epsg()
 
     @property
     def semi_major_axis(self):
@@ -200,14 +225,14 @@ class CRS:
 
         :returns: ``("", "")`` when not available
         """
-        if self._epsg is not None:
+        if self._epsg:
             return ("EPSG", self._epsg)
 
         if (r := self._crs.to_authority()) is not None:
             name, code = r
             try:
                 return (name, int(code))
-            except ValueError:
+            except ValueError:  # pragma: nocover
                 return (name, code)
 
         return ("", "")
@@ -231,8 +256,11 @@ class CRS:
         if self._crs is other._crs:
             return True
 
-        if self.epsg is not None and other.epsg is not None:
-            return self.epsg == other.epsg
+        if self._epsg and other._epsg:
+            return self._epsg == other._epsg
+
+        if self._str == other._str:
+            return True
 
         return self._crs == other._crs
 
@@ -289,10 +317,10 @@ class CRS:
         """
 
         # pylint: disable=protected-access
-        transform = _make_crs_transform(self._crs, other._crs, always_xy=always_xy)
+        tr = _make_crs_transform(self._crs, other._crs, always_xy=always_xy)
 
-        def result(x, y):
-            rx, ry = transform(x, y)  # pylint: disable=unpacking-non-sequence
+        def result(x, y, **kw):
+            rx, ry = tr.transform(x, y, **kw)  # pylint: disable=unpacking-non-sequence
 
             if not isinstance(rx, numpy.ndarray) or not isinstance(ry, numpy.ndarray):
                 return (rx, ry)
@@ -307,6 +335,53 @@ class CRS:
     def __dask_tokenize__(self):
         return ("odc.geo.crs.CRS", str(self))
 
+    @staticmethod
+    def utm(
+        x: Union[float, int, XY[float], "geom.Geometry", "geom.BoundingBox"],
+        y: Optional[float] = None,
+        /,
+        datum_name: str = "WGS 84",
+    ) -> "CRS":
+        """
+        Construct appropriate UTM CRS for a given point.
+
+        Uses CRS database query methods from :py:mod:`pyproj` to locate
+        appropriate UTM CRS.
+
+        :params datum_name: The name of the datum in the CRS name ('NAD27', 'NAD83', 'WGS 84', ...)
+        """
+        # pylint: disable=import-outside-toplevel,no-name-in-module
+        from pyproj.database import query_utm_crs_info
+
+        from . import geom
+
+        if isinstance(x, geom.BoundingBox):
+            _bbox = x
+        elif isinstance(x, geom.Geometry):
+            if x.crs is not None:
+                _bbox = x.to_crs("epsg:4326").boundingbox
+            else:
+                # assume already in lon/lat
+                _bbox = x.boundingbox
+        elif isinstance(x, (float, int)):
+            if y is None:
+                y = 0.0
+            _bbox = geom.BoundingBox(x, y, x, y)
+        else:
+            x, y = x.xy
+            _bbox = geom.BoundingBox(x, y, x, y)
+
+        return _pick_best_crs(
+            _bbox.polygon,
+            [
+                CRS(f"{info.auth_name}:{info.code}")
+                for info in query_utm_crs_info(
+                    datum_name=datum_name,
+                    area_of_interest=_bbox.aoi,
+                )
+            ],
+        )
+
 
 class CRSMismatchError(ValueError):
     """
@@ -321,26 +396,47 @@ class CRSMismatchError(ValueError):
 @overload
 def norm_crs(crs: SomeCRS) -> CRS: ...
 @overload
-def norm_crs(crs: None) -> None: ...
+def norm_crs(crs: SomeCRS, ctx: Any) -> CRS: ...
+@overload
+def norm_crs(crs: Union[None, Unset]) -> None: ...
+@overload
+def norm_crs(crs: Union[None, Unset], ctx: Any) -> None: ...
 # fmt: on
 
 
-def norm_crs(crs: MaybeCRS) -> Optional[CRS]:
+def norm_crs(crs: MaybeCRS, ctx=None) -> Optional[CRS]:
     """Normalise CRS representation."""
     if isinstance(crs, CRS):
         return crs
-    if crs is None:
+    if crs is None or isinstance(crs, Unset):
         return None
+    if isinstance(crs, str):
+        _txt = crs.lower()
+        if _txt.startswith("utm"):
+            assert ctx is not None
+
+            utm_crs = CRS.utm(ctx)
+            if _txt == "utm":
+                return utm_crs
+
+            utm_zone = utm_crs.proj.utm_zone
+            epsg = utm_crs.epsg
+            assert utm_zone is not None
+            assert epsg is not None
+            if _txt == "utm-n" and utm_zone.endswith("S"):
+                utm_crs = CRS(epsg - 100)
+            elif _txt == "utm-s" and utm_zone.endswith("N"):
+                utm_crs = CRS(epsg + 100)
+            return utm_crs
     return CRS(crs)
 
 
-def norm_crs_or_error(crs: MaybeCRS) -> CRS:
+def norm_crs_or_error(crs: MaybeCRS, ctx=None) -> CRS:
     """Normalise CRS representation, raise error if input is ``None``."""
-    if isinstance(crs, CRS):
-        return crs
-    if crs is None:
+    _crs = norm_crs(crs, ctx=ctx)
+    if _crs is None:
         raise ValueError("Expect valid CRS")
-    return CRS(crs)
+    return _crs
 
 
 def crs_units_per_degree(
@@ -380,5 +476,27 @@ def crs_units_per_degree(
     return xy.length / step
 
 
+def _pick_best_crs(poly: "geom.Geometry", crs_candidates: List[CRS]) -> CRS:
+    # pylint: disable=import-outside-toplevel
+    from . import geom
+
+    def overlap_pct(crs: CRS) -> float:
+        crs_region = crs.valid_region
+        if crs_region is None:
+            return 1  # pragma: nocover
+        return (crs_region & poly).area / poly.area
+
+    if len(crs_candidates) < 1:
+        raise ValueError("No candidate CRSs found")
+
+    if len(crs_candidates) > 1 and poly.area > 1e-9:
+        if poly.crs is None:
+            poly = geom.Geometry(poly.geom, "epsg:4326")
+
+        crs_candidates = sorted(crs_candidates, key=overlap_pct, reverse=True)
+
+    return crs_candidates[0]
+
+
 if TYPE_CHECKING:
     from . import geom  # pragma: no cover
