@@ -7,13 +7,23 @@ import itertools
 import math
 from collections import OrderedDict, namedtuple
 from enum import Enum
-from typing import Dict, Iterable, List, Literal, Optional, Tuple, Union
+from typing import (
+    Dict,
+    Iterable,
+    List,
+    Literal,
+    Mapping,
+    Optional,
+    Sequence,
+    Tuple,
+    Union,
+)
 
 import numpy
 from affine import Affine
 
 from . import geom
-from .crs import CRS, CRSMismatchError, MaybeCRS, SomeCRS, norm_crs
+from .crs import CRS, MaybeCRS, SomeCRS, norm_crs
 from .geom import BoundingBox, Geometry, bbox_intersection, bbox_union
 from .math import (
     clamp,
@@ -24,11 +34,19 @@ from .math import (
     snap_grid,
     split_translation,
 )
-from .roi import Tiles as RoiTiles
-from .roi import align_up, roi_boundary, roi_normalise, roi_shape
+from .roi import (
+    RoiTiles,
+    align_up,
+    clip_tiles,
+    roi_boundary,
+    roi_normalise,
+    roi_shape,
+    roi_tiles,
+)
 from .types import (
+    ROI,
     XY,
-    Index2d,
+    Chunks2d,
     MaybeInt,
     NormalizedROI,
     OutlineMode,
@@ -38,7 +56,7 @@ from .types import (
     SomeResolution,
     SomeShape,
     Unset,
-    iyx_,
+    func2map,
     res_,
     shape_,
     xy_,
@@ -51,8 +69,13 @@ class AnchorEnum(Enum):
     """
 
     EDGE = 0
+    """Snap pixel edges to multiples of pixel size."""
+
     CENTER = 1
+    """Snap pixel centers to multiples of pixel size."""
+
     FLOATING = 2
+    """Turn off pixel snapping."""
 
 
 GeoboxAnchor = Union[AnchorEnum, XY[float]]
@@ -203,7 +226,7 @@ class GeoBoxBase:
         """
         assert self.crs is not None
         ext = self.extent
-        if buffer > 0:
+        if buffer != 0:
             buffer = buffer * max(*self.resolution.xy)
             ext = ext.buffer(buffer)
 
@@ -304,7 +327,10 @@ class GeoBoxBase:
         return (shape_((ny, nx)), A)
 
     def compute_zoom_to(
-        self, shape: Union[SomeShape, int, float]
+        self,
+        shape: Union[SomeShape, int, float, None] = None,
+        *,
+        resolution: Optional[SomeResolution] = None,
     ) -> Tuple[Shape2d, Affine]:
         """
         Change GeoBox shape.
@@ -314,6 +340,14 @@ class GeoBoxBase:
         :returns:
           GeoBox covering the same region but with different number of pixels and therefore resolution.
         """
+        if shape is None:
+            if resolution is None:
+                raise ValueError("Have to supply shape or resolution")
+            new_geobox = GeoBox.from_bbox(
+                self.boundingbox, resolution=resolution, tight=True
+            )
+            return new_geobox.shape, new_geobox.affine
+
         if isinstance(shape, (int, float)):
             nmax = max(*self._shape)
             return self.compute_zoom_out(nmax / shape)
@@ -372,6 +406,9 @@ class GeoBoxBase:
             n, padding=padding, with_edges=with_edges, offset=offset
         )
 
+    def __getitem__(self, roi) -> "GeoBoxBase":
+        raise NotImplementedError()
+
 
 class GeoBox(GeoBoxBase):
     """
@@ -829,7 +866,12 @@ class GeoBox(GeoBoxBase):
         _shape, _affine = self.compute_zoom_out(factor)
         return GeoBox(_shape, _affine, self._crs)
 
-    def zoom_to(self, shape: Union[SomeShape, int, float]) -> "GeoBox":
+    def zoom_to(
+        self,
+        shape: Union[SomeShape, int, float, None] = None,
+        *,
+        resolution: Optional[SomeResolution] = None,
+    ) -> "GeoBox":
         """
         Change GeoBox shape.
 
@@ -838,7 +880,7 @@ class GeoBox(GeoBoxBase):
         :returns:
           GeoBox covering the same region but with different number of pixels and therefore resolution.
         """
-        _shape, _affine = self.compute_zoom_to(shape)
+        _shape, _affine = self.compute_zoom_to(shape, resolution=resolution)
         return GeoBox(_shape, _affine, self._crs)
 
     def flipy(self) -> "GeoBox":
@@ -1125,9 +1167,14 @@ def zoom_out(gbox: GeoBox, factor: float) -> GeoBox:
     return gbox.zoom_out(factor)
 
 
-def zoom_to(gbox: GeoBox, shape: SomeShape) -> GeoBox:
+def zoom_to(
+    gbox: GeoBox,
+    shape: Union[SomeShape, int, float, None] = None,
+    *,
+    resolution: Optional[SomeResolution] = None,
+) -> GeoBox:
     """Alias for :py:meth:`odc.geo.geobox.GeoBox.zoom_to`."""
-    return gbox.zoom_to(shape)
+    return gbox.zoom_to(shape, resolution=resolution)
 
 
 def rotate(gbox: GeoBox, deg: float) -> GeoBox:
@@ -1143,7 +1190,15 @@ def affine_transform_pix(gbox: GeoBox, transform: Affine) -> GeoBox:
 class GeoboxTiles:
     """Partition GeoBox into sub geoboxes."""
 
-    def __init__(self, box: GeoBox, tile_shape: SomeShape):
+    __slots__ = ("_gbox", "_tiles")
+
+    def __init__(
+        self,
+        box: GeoBoxBase,
+        tile_shape: Union[SomeShape, Chunks2d, None],
+        *,
+        _tiles: Optional[RoiTiles] = None,
+    ):
         """
         Construct from a :py:class:`~odc.geo.GeoBox`.
 
@@ -1151,11 +1206,14 @@ class GeoboxTiles:
         :param tile_shape: Shape of sub-tiles in pixels ``(rows, cols)``
         """
         self._gbox = box
-        self._tiles = RoiTiles(box.shape, tile_shape)
-        self._cache: Dict[Index2d, GeoBox] = {}
+        if _tiles is not None:
+            self._tiles = _tiles
+        else:
+            assert tile_shape is not None
+            self._tiles = roi_tiles(box.shape, tile_shape)
 
     @property
-    def base(self) -> GeoBox:
+    def base(self) -> GeoBoxBase:
         """Access base Geobox"""
         return self._gbox
 
@@ -1186,7 +1244,31 @@ class GeoboxTiles:
         """
         return self._tiles.tile_shape(idx)
 
-    def __getitem__(self, idx: SomeIndex2d) -> GeoBox:
+    @property
+    def chunks(self) -> Chunks2d:
+        return self._tiles.chunks
+
+    def _crop(self, roi: ROI) -> "GeoboxTiles":
+        gbox_new = self.base[self._tiles[roi]]
+        return GeoboxTiles(gbox_new, (0, 0), _tiles=self._tiles.crop(roi))
+
+    def clip(
+        self, selection: Sequence[Tuple[int, int]]
+    ) -> Tuple["GeoboxTiles", List[Tuple[int, int]]]:
+        """
+        Crop to a set of tiles.
+
+        Returns cropped version of :py:class:`GeoboxTiles` and a list of
+        tile coordinates in the new, cropped space.
+        """
+        tiles, roi, new_idx = clip_tiles(self._tiles, selection)
+        return GeoboxTiles(self[roi], None, _tiles=tiles), new_idx
+
+    @property
+    def crop(self) -> Mapping[ROI, "GeoboxTiles"]:
+        return func2map(self._crop)
+
+    def __getitem__(self, idx: Union[SomeIndex2d, ROI]) -> GeoBoxBase:
         """
         Lookup tile by index, index is in matrix access order: ``(row, col)``.
 
@@ -1194,13 +1276,7 @@ class GeoboxTiles:
         :returns: GeoBox of a tile
         :raises: IndexError when index is outside of ``[(0,0) -> .shape)``
         """
-        idx = iyx_(idx)
-        sub_gbox = self._cache.get(idx, None)
-        if sub_gbox is not None:
-            return sub_gbox
-
-        roi = self._tiles[idx]
-        return self._cache.setdefault(idx, self._gbox[roi])
+        return self._gbox[self._tiles[idx]]
 
     def range_from_bbox(self, bbox: BoundingBox) -> Tuple[range, range]:
         """
@@ -1209,40 +1285,78 @@ class GeoboxTiles:
         Compute rows and columns overlapping with a given :py:class:`~odc.geo.geom.BoundingBox`.
         """
 
-        def clamped_range(v1: float, v2: float, N: int) -> range:
-            _in = clamp(math.floor(v1), 0, N)
-            _out = clamp(math.ceil(v2), 0, N)
-            return range(_in, _out)
+        if bbox.crs is not None:
+            bbox = self._gbox.project(bbox.polygon).boundingbox
 
-        if bbox.crs != self._gbox.crs:
-            raise CRSMismatchError()
+        def _clamp(span: Tuple[float, float], N: int):
+            a1, a2 = span
+            a1 = int(clamp(math.floor(a1), 0, N - 1))
+            a2 = int(clamp(math.ceil(a2), 1, N)) - 1
+            return a1, a2
 
-        sy, sx = self._tiles.tile_shape((0, 0)).yx
-        A = Affine.scale(1.0 / sx, 1.0 / sy) * (~self._gbox.transform)
-        # A maps from X,Y in meters to chunk index
-        bbox = bbox.transform(A)
+        NY, NX = self._gbox.shape.yx
+        x1, x2 = _clamp(bbox.range_x, NX)
+        y1, y2 = _clamp(bbox.range_y, NY)
 
-        NY, NX = self._tiles.shape.yx
-        xx = clamped_range(bbox.left, bbox.right, NX)
-        yy = clamped_range(bbox.bottom, bbox.top, NY)
-        return (yy, xx)
+        y1, x1 = self._tiles.locate((y1, x1))
+        y2, x2 = self._tiles.locate((y2, x2))
+
+        return range(y1, y2 + 1), range(x1, x2 + 1)
 
     def tiles(self, polygon: Geometry) -> Iterable[Tuple[int, int]]:
         """Return tile indexes overlapping with a given geometry."""
         target_crs = self._gbox.crs
         poly = polygon
         if target_crs is not None and poly.crs != target_crs:
-            poly = poly.to_crs(target_crs)
+            poly = poly.to_crs(target_crs, check_and_fix=True)
 
         yy, xx = self.range_from_bbox(poly.boundingbox)
         for idx in itertools.product(yy, xx):
             gbox = self[idx]
-            if gbox.extent.intersects(poly):
+            if not poly.disjoint(gbox.extent):
                 yield idx
 
+    def grid_intersect(
+        self, src: "GeoboxTiles"
+    ) -> Dict[Tuple[int, int], List[Tuple[int, int]]]:
+        """
+        Figure out tile to tile overlap graph between two grids.
+
+        For every tile in this :py:class:`GeoboxTiles` find every tile in ``other`` that
+        intersects with this ``tile``.
+        """
+        if src.base.crs == self.base.crs:
+            src_footprint = src.base.extent
+        else:
+            # compute "robust" source footprint in CRS of self via espg:4326
+            src_footprint = (
+                src.base.footprint(4326, 2) & self.base.footprint(4326, 2)
+            ).to_crs(self.base.crs)
+
+        xy_chunks_with_data = list(self.tiles(src_footprint))
+        deps: Dict[Tuple[int, int], List[Tuple[int, int]]] = {}
+
+        for idx in xy_chunks_with_data:
+            geobox = self[idx]
+            deps[idx] = list(src.tiles(geobox.extent))
+
+        return deps
+
     def __dask_tokenize__(self):
         return (
             "odc.geo.geobox.GeoboxTiles",
-            *self._tiles.shape.yx,
             *self._gbox.__dask_tokenize__()[1:],
+            *self._tiles.__dask_tokenize__()[1:],
         )
+
+    def __str__(self):
+        return str(self.roi)
+
+    def __eq__(self, __value: object) -> bool:
+        if not isinstance(__value, GeoboxTiles):
+            return False
+        if self is __value:
+            return True
+        return self._tiles == __value._tiles and self._gbox == __value._gbox
+
+    __repr__ = __str__
