@@ -15,6 +15,7 @@ from typing import (
     Iterable,
     Iterator,
     List,
+    Literal,
     Optional,
     Sequence,
     Tuple,
@@ -423,7 +424,7 @@ def densify(coords: CoordList, resolution: float) -> CoordList:
 
 
 def _clone_shapely_geom(geom: base.BaseGeometry) -> base.BaseGeometry:
-    return type(geom)(geom)
+    return geometry.shape(geom)
 
 
 class Geometry(SupportsCoords[float]):
@@ -472,6 +473,8 @@ class Geometry(SupportsCoords[float]):
     @wrap_shapely
     def contains(self, other: "Geometry") -> bool: return self.contains(other)
     @wrap_shapely
+    def covers(self, other: "Geometry") -> bool: return self.covers(other)
+    @wrap_shapely
     def crosses(self, other: "Geometry") -> bool: return self.crosses(other)
     @wrap_shapely
     def disjoint(self, other: "Geometry") -> bool: return self.disjoint(other)
@@ -502,6 +505,8 @@ class Geometry(SupportsCoords[float]):
 
     def _repr_svg_(self) -> str: return self.geom._repr_svg_()
 
+    @property
+    def geom_type(self) -> str: return self.geom.geom_type
     @property
     def type(self) -> str: return self.geom.type
     @property
@@ -554,22 +559,28 @@ class Geometry(SupportsCoords[float]):
         """
 
         def segmentize_shapely(geom: base.BaseGeometry) -> base.BaseGeometry:
-            if geom.type in ["Point", "MultiPoint"]:
-                return type(geom)(geom)  # clone without changes
+            if geom.geom_type in ["Point", "MultiPoint"]:
+                return _clone_shapely_geom(geom)  # clone without changes
 
-            if geom.type in ["GeometryCollection", "MultiPolygon", "MultiLineString"]:
+            if geom.geom_type in [
+                "GeometryCollection",
+                "MultiPolygon",
+                "MultiLineString",
+            ]:
                 return type(geom)([segmentize_shapely(g) for g in geom.geoms])
 
-            if geom.type in ["LineString", "LinearRing"]:
+            if geom.geom_type in ["LineString", "LinearRing"]:
                 return type(geom)(densify(list(geom.coords), resolution))
 
-            if geom.type == "Polygon":
+            if geom.geom_type == "Polygon":
                 return geometry.Polygon(
                     densify(list(geom.exterior.coords), resolution),
                     [densify(list(i.coords), resolution) for i in geom.interiors],
                 )
 
-            raise ValueError(f"unknown geometry type {geom.type}")  # pragma: no cover
+            raise ValueError(
+                f"unknown geometry type {geom.geom_type}"
+            )  # pragma: no cover
 
         return Geometry(segmentize_shapely(self.geom), self.crs)
 
@@ -625,8 +636,10 @@ class Geometry(SupportsCoords[float]):
     def to_crs(
         self,
         crs: SomeCRS,
-        resolution: Optional[float] = None,
+        resolution: Union[float, Literal["auto"], None] = None,
         wrapdateline: bool = False,
+        *,
+        check_and_fix: bool = False,
     ) -> "Geometry":
         """
         Convert geometry to a different Coordinate Reference System.
@@ -652,6 +665,10 @@ class Geometry(SupportsCoords[float]):
            Attempt to gracefully handle geometry that intersects the dateline when converting to
            geographic projections. Currently only works in few specific cases (source CRS is smooth
            over the dateline).
+
+        :param check_and_fix:
+           When ``True`` remove vertices that didn't project cleanly and apply ``.buffer(0)`` is geometry
+           is not valid after projection.
         """
         crs = norm_crs_or_error(crs, self)
         if self.crs == crs:
@@ -660,20 +677,31 @@ class Geometry(SupportsCoords[float]):
         if self.crs is None:
             raise ValueError("Cannot project geometries without CRS")
 
+        if resolution == "auto":
+            resolution = _auto_resolution(self)
+
         if resolution is not None and math.isfinite(resolution):
             geom = self.segmented(resolution)
         else:
             geom = self
 
+        def maybe_fix(g: Geometry) -> Geometry:
+            if not check_and_fix or g.is_valid:
+                return g
+            g = g.dropna()
+            if g.geom_type in ["Polygon", "MultiPolygon"] and not g.is_valid:
+                g = g.buffer(0)
+            return g
+
         eps = 1e-4
         if wrapdateline and crs.geographic:
             # TODO: derive precision from resolution by converting to degrees
             precision = 0.1
             chopped = chop_along_antimeridian(geom, precision)
-            chopped_lonlat = chopped._to_crs(crs)
+            chopped_lonlat = maybe_fix(chopped._to_crs(crs))
             return clip_lon180(chopped_lonlat, eps)
 
-        return geom._to_crs(crs)
+        return maybe_fix(geom._to_crs(crs))
 
     def assign_crs(self, crs: MaybeCRS) -> "Geometry":
         """
@@ -711,7 +739,7 @@ class Geometry(SupportsCoords[float]):
 
         :return: GeoJSON Feature dictionary
         """
-        if self.type == "GeometryCollection":
+        if self.geom_type == "GeometryCollection":
             return {
                 "type": "FeatureCollection",
                 "features": [
@@ -792,7 +820,7 @@ class Geometry(SupportsCoords[float]):
     @property
     def is_multi(self) -> bool:
         """True for multi-geometry types."""
-        t = self.type
+        t = self.geom_type
         return t.startswith("Multi") or t == "GeometryCollection"
 
     def __rmul__(self, A: Affine) -> "Geometry":
@@ -838,7 +866,7 @@ class Geometry(SupportsCoords[float]):
         if self.is_multi:
             return "".join([g.svg_path(ndecimal) for g in self.geoms])
 
-        if self.type == "Polygon":
+        if self.geom_type == "Polygon":
             return "".join(
                 [render_ring(g, close=True) for g in [self.exterior, *self.interiors]]
             )
@@ -850,7 +878,7 @@ class Geometry(SupportsCoords[float]):
         Keep only those points for which `pred(x,y) is True`.
         """
 
-        if self.type == "Polygon":
+        if self.geom_type == "Polygon":
             pts = [(x, y) for x, y in self.exterior.points if pred(x, y)]
             # prune inner polygon points
             inners = [
@@ -861,18 +889,20 @@ class Geometry(SupportsCoords[float]):
             inners = [ring for ring in inners if len(ring) >= 3]
             return polygon(pts, self.crs, *inners)
 
-        if self.type in ("LinearRing", "LineString"):
+        if self.geom_type in ("LinearRing", "LineString"):
             pts = [(x, y) for x, y in self.points if pred(x, y)]
+            if len(pts) == 1:
+                pts = []  # need at least 2 points for this type
             _geom = type(self.geom)(pts)
             return Geometry(_geom, self.crs)
 
-        if self.type == "Point":
+        if self.geom_type == "Point":
             x, y = self.coords[0]
             if pred(x, y):
                 return self
             return Geometry(geometry.Point(), self.crs)
 
-        if self.type == "MultiPoint":
+        if self.geom_type == "MultiPoint":
             pts = [(x, y) for x, y in [pt.points[0] for pt in self.geoms] if pred(x, y)]
             return multipoint(pts, self.crs)
 
@@ -934,6 +964,8 @@ def projected_lon(
         for x, y in zip(xx_, yy_)
         if math.isfinite(x) and math.isfinite(y)
     ]
+    if len(pts) < 2:
+        return line([], crs)
     return line(pts, crs)
 
 
@@ -968,7 +1000,7 @@ def clip_lon180(geom: Geometry, tol=1e-6) -> Geometry:
         clip = _pick_clip(xx)
         return _clip_180(xx, clip), yy
 
-    if geom.type.startswith("Multi"):
+    if geom.geom_type.startswith("Multi"):
         return multigeom(g.transform(transformer) for g in geom.geoms)
 
     return geom.transform(transformer)
@@ -1118,10 +1150,11 @@ def sides(poly: Geometry) -> Iterable[Geometry]:
 
 
 def _multigeom(geoms: List[base.BaseGeometry]) -> base.BaseGeometry:
-    src_type = {g.type for g in geoms}
+    src_type = {g.geom_type for g in geoms}
     if len(src_type) > 1:
         return geometry.GeometryCollection(geoms)
 
+    geoms = [g for g in geoms if not g.is_empty]
     src_type = src_type.pop()
     if src_type == "Polygon":
         return geometry.MultiPolygon(geoms)
@@ -1255,10 +1288,12 @@ def bbox_intersection(bbs: Iterable[BoundingBox]) -> BoundingBox:
 
 
 def lonlat_bounds(
-    geom: Geometry, mode: str = "safe", resolution: Optional[float] = None
+    geom: Geometry,
+    mode: str = "safe",
+    resolution: Union[float, None, Literal["auto"]] = None,
 ) -> BoundingBox:
     """
-    Return the bounding box of a geometry.
+    Return the bounding box of a geometry in lon/lat.
 
     :param geom:
        Geometry in any projection
@@ -1275,10 +1310,13 @@ def lonlat_bounds(
     if geom.crs.geographic:
         return geom.boundingbox
 
+    if resolution == "auto":
+        resolution = _auto_resolution(geom)
+
     if resolution is not None and math.isfinite(resolution):
         geom = geom.segmented(resolution)
 
-    bbox = geom.to_crs("EPSG:4326").boundingbox
+    bbox = geom.to_crs("EPSG:4326", check_and_fix=True).boundingbox
 
     xx_range = bbox.range_x
     if mode == "safe":
@@ -1298,7 +1336,7 @@ def lonlat_bounds(
             if span_x_ < bbox.span_x:
                 xx_range = xx_range_
 
-    return BoundingBox.from_xy(xx_range, bbox.range_y)
+    return BoundingBox.from_xy(xx_range, bbox.range_y, crs=4326)
 
 
 def mid_longitude(geom: Geometry) -> float:
@@ -1307,3 +1345,8 @@ def mid_longitude(geom: Geometry) -> float:
     """
     ((lon,), _) = geom.centroid.to_crs("epsg:4326").xy
     return lon
+
+
+def _auto_resolution(g: Geometry) -> float:
+    # aim for ~100 points per side of a square
+    return math.sqrt(g.area) * 4 / 100
