@@ -31,7 +31,7 @@ from .roi import (
     roi_is_empty,
     scaled_up_roi,
 )
-from .types import XY, SomeShape, res_, shape_, xy_
+from .types import XY, SomeResolution, SomeShape, res_, shape_, xy_
 
 
 class PointTransform(Protocol):
@@ -183,7 +183,10 @@ class ReprojectInfo:
     """
 
     scale: float
-    """Scale change as a single number."""
+    """Scale change as a single number. (the min of scale2)"""
+
+    scale2: XY[float]
+    """Full 2D Scale change as an XY."""
 
     transform: PointTransform
     """Mapping from src pixels to destination pixels."""
@@ -387,7 +390,7 @@ def _can_paste(
     if any(abs(abs(s) - 1) > stol for s in (sx, sy)):  # not equal scaling across axis?
         return False, "sx!=sy, probably"
 
-    # Check is sub-pixel translation is within bounds
+    # Check if sub-pixel translation within bounds
     if not all(is_almost_int(t, ttol) for t in (tx, ty)):
         return False, "sub-pixel translation"
 
@@ -501,12 +504,14 @@ def compute_reproject_roi(
             read_shrink = _pick_read_scale(scale)
         else:
             scale = 0
+            scale2 = XY(x=0, y=0)
             read_shrink = 1
 
         return ReprojectInfo(
             roi_src=roi_src,
             roi_dst=roi_dst,
             scale=scale,
+            scale2=scale2,
             paste_ok=False,
             read_shrink=read_shrink,
             transform=tr,
@@ -546,6 +551,7 @@ def compute_reproject_roi(
         roi_src=roi_src,
         roi_dst=roi_dst,
         scale=scale,
+        scale2=scale2,
         paste_ok=paste_ok,
         read_shrink=read_shrink,
         transform=tr,
@@ -556,7 +562,7 @@ def compute_output_geobox(
     gbox: Union[GeoBox, GCPGeoBox],
     crs: SomeCRS,
     *,
-    resolution: Literal["auto", "fit", "same"] = "auto",
+    resolution: Union[SomeResolution, Literal["auto", "fit", "same"]] = "auto",
     tight: bool = False,
 ) -> GeoBox:
     """
@@ -577,6 +583,7 @@ def compute_output_geobox(
        * "fit" use center pixel to determine scale change between the two
        * | "auto" is to use the same resolution on the output if CRS units are the same
          |  between the source and destination and otherwise use "fit"
+       * Else resolution in the units of the output crs
 
     :param tight:
       By default output pixel grid is adjusted to align pixel edges to X/Y axis, suppling
@@ -593,6 +600,14 @@ def compute_output_geobox(
     bbox = gbox.footprint(crs, buffer=0.9, npoints=100).boundingbox
     dst_crs = bbox.crs
     assert dst_crs is not None
+
+    if (
+        dst_crs == src_crs
+        and resolution in ("auto", "same")
+        and isinstance(gbox, GeoBox)
+    ):
+        return gbox
+
     same_units = src_crs.units == dst_crs.units
 
     if resolution == "same":
@@ -610,15 +625,18 @@ def compute_output_geobox(
         #  Y = tr(X) => Y ~= s*X + t
         # where X is in `dst_` and Y is in `gbox`
         # .. so a better fit resolution is `_dst.res / s`
-        # .. but we want square pixels so pick average between x/y
+        # .. but we want square pixels, so pick average between x and y
         sx, sy = get_scale_at_point(xy_(0.5, 0.5), native_pix_transform(dst_, cp)).xy
 
         # always produces square pixels on output with inverted Y axis
         avg_res = (abs(dst_.resolution.x / sx) + abs(dst_.resolution.y / sy)) / 2
         res = res_(avg_res)
     else:
-        raise ValueError(
-            f"Resolution ought to be one of: same,auto,fit, not '{resolution}'"
-        )
+        if isinstance(resolution, str):
+            raise ValueError(
+                f"Resolution ought to be one of: same,auto,fit, not '{resolution}'"
+            )
+
+        res = res_(resolution)
 
     return GeoBox.from_bbox(bbox, dst_crs, resolution=res, tight=tight)
