@@ -10,12 +10,14 @@ will have an ROI that can be constructed with :py:func:`numpy.s_` like this: ``s
 """
 import math
 from collections import abc
-from typing import Optional, Tuple, Union, overload
+from typing import List, Optional, Protocol, Sequence, Tuple, Union, overload
 
 import numpy as np
 
 from .math import align_down, align_up, edge_index
 from .types import (
+    ROI,
+    Chunks2d,
     NdROI,
     NormalizedROI,
     NormalizedSlice,
@@ -53,6 +55,71 @@ class WindowFromSlice:
 w_ = WindowFromSlice()
 
 
+class RoiTiles(Protocol):
+    """
+    Abstraction for 2d slice/shape/chunks lookup.
+    """
+
+    def __getitem__(self, idx: Union[SomeIndex2d, ROI]) -> Tuple[slice, slice]:
+        ...
+
+    def crop(self, roi: ROI) -> "RoiTiles":
+        ...
+
+    def tile_shape(self, idx: SomeIndex2d) -> Shape2d:
+        ...
+
+    @property
+    def shape(self) -> Shape2d:
+        ...
+
+    @property
+    def base(self) -> Shape2d:
+        ...
+
+    @property
+    def chunks(self) -> Chunks2d:
+        ...
+
+    def locate(self, pix: SomeIndex2d) -> Tuple[int, int]:
+        ...
+
+    def __dask_tokenize__(self):
+        ...
+
+
+def norm_slice_2d(
+    idx: Union[SomeIndex2d, ROI], shape: Tuple[int, int]
+) -> NormalizedROI:
+    if isinstance(idx, tuple):
+        return roi_normalise(idx, shape)
+    return roi_normalise(iyx_(idx).yx, shape)
+
+
+def _fmt_shape(shape):
+    n1, n2 = shape.yx
+    if max(n1, n2) > 10_000:
+        return f"{n1:_d}x{n2:_d}"
+    return f"{n1:d}x{n2:d}"
+
+
+def clip_tiles(
+    tiles: RoiTiles, selection: Sequence[Tuple[int, int]]
+) -> Tuple["RoiTiles", Tuple[slice, slice], List[Tuple[int, int]]]:
+    """
+    Compute cropped version of tiles from a selection of tile indexes.
+
+    :return: Cropped RoiTiles object, crop that was applied and tile indexes
+             changed to the cropped address space.
+    """
+    ii = np.asarray(selection)
+    y1, x1 = ii.min(axis=0).tolist()
+    y2, x2 = ii.max(axis=0).tolist()
+    roi = np.s_[y1 : y2 + 1, x1 : x2 + 1]
+    sel_new = [(y - y1, x - x1) for y, x in selection]
+    return tiles.crop(roi), roi, sel_new
+
+
 class Tiles:
     """
     Partition box into tiles.
@@ -70,18 +137,24 @@ class Tiles:
         )
         self._shape = shape_((ny, nx))
 
-    def __getitem__(self, idx: SomeIndex2d) -> NormalizedROI:
-        def _slice(i, N, n) -> NormalizedSlice:
-            _in = i * n
-            if 0 <= _in < N:
-                return slice(_in, min(_in + n, N))
+    def crop(self, roi: ROI) -> "Tiles":
+        base_shape = roi_shape(self[roi])
+        return Tiles(base_shape, self._tile_shape)
+
+    def __getitem__(self, idx: Union[SomeIndex2d, ROI]) -> Tuple[slice, slice]:
+        def _slice(i: NormalizedSlice, N: int, n: int) -> slice:
+            _in = i.start * n
+            _out = i.stop * n
+
+            if 0 <= _in < N and _out < N + n:
+                return slice(_in, min(_out, N))
             raise IndexError(f"Index {idx} is out of range")
 
-        idx = iyx_(idx)
+        idx = norm_slice_2d(idx, self._shape.yx)
 
         ir, ic = (
             _slice(i, N, n)
-            for i, N, n in zip(idx.yx, self._base_shape.yx, self._tile_shape.yx)
+            for i, N, n in zip(idx, self._base_shape.yx, self._tile_shape.yx)
         )
         return (ir, ic)
 
@@ -89,13 +162,15 @@ class Tiles:
         """
         Query shape for a given tile.
 
-        :param idx: ``(row, col)`` chunk index
+        :param idx: ``(row, col)`` chunk index, supports numpy style indexing.
         :returns: shape of a tile (edge tiles might be smaller)
         :raises: :py:class:`IndexError` when index is outside of ``[(0,0) -> .shape)``.
         """
         idx = iyx_(idx)
 
         def _sz(i: int, n: int, tile_sz: int, total_sz: int) -> int:
+            if i < 0:  # numpy style index from the right
+                i = n + i
             if 0 <= i < n - 1:  # not edge tile
                 return tile_sz
             if i == n - 1:  # edge tile
@@ -118,6 +193,154 @@ class Tiles:
         """Base shape."""
         return self._base_shape
 
+    @property
+    def chunks(self) -> Chunks2d:
+        """Dask compatible chunk rerpesentation."""
+        NY, NX = self.shape.yx
+        ny, nx = self.tile_shape((0, 0)).yx
+        ny_, nx_ = self.tile_shape((NY - 1, NX - 1))
+
+        return (
+            (ny,) * (NY - 1) + (ny_,),
+            (nx,) * (NX - 1) + (nx_,),
+        )
+
+    def locate(self, pix: SomeIndex2d) -> Tuple[int, int]:
+        """Tile index from pixel coordinate."""
+        NY, NX = self._base_shape.yx
+        y, x = iyx_(pix).yx
+        if y < 0 or y >= NY or x < 0 or x >= NX:
+            raise IndexError()
+        ny, nx = self.tile_shape((0, 0)).yx
+        return (y // ny, x // nx)
+
+    def __dask_tokenize__(self):
+        return (
+            "odc.geo.roi.Tiles",
+            *self._shape,
+            *self._tile_shape,
+        )
+
+    def __str__(self) -> str:
+        b1, b2, b3 = map(_fmt_shape, [self._shape, self._tile_shape, self._base_shape])
+        return f"Tiles: {b1}|{b2}px => {b3}px"
+
+    def __repr__(self) -> str:
+        return self.__str__()
+
+    def __eq__(self, __value: object) -> bool:
+        if __value is self:
+            return True
+        if not isinstance(__value, Tiles):
+            return False
+        return (
+            self._base_shape == __value._base_shape
+            and self._tile_shape == __value._tile_shape
+        )
+
+
+class VariableSizedTiles:
+    """
+    Partition box into tiles of varying sizes.
+
+    Turns ``row, col`` index into a 2d ROI of the original box.
+    """
+
+    __slots__ = ("_offsets",)
+
+    def __init__(self, chunks: Chunks2d) -> None:
+        self._offsets = tuple(
+            np.asarray([0, *idx], dtype="int32").cumsum(dtype="int32") for idx in chunks
+        )
+
+    def crop(self, roi: ROI) -> "VariableSizedTiles":
+        roi = roi_normalise(roi, self.shape.yx)
+        y, x = (ch[s.start : s.stop] for ch, s in zip(self.chunks, roi))
+        return VariableSizedTiles((y, x))
+
+    def __getitem__(self, idx: Union[SomeIndex2d, ROI]) -> Tuple[slice, slice]:
+        idx = norm_slice_2d(idx, self.shape.yx)
+        y, x = (
+            slice(int(a[i.start]), int(a[i.stop])) for a, i in zip(self._offsets, idx)
+        )
+        return (y, x)
+
+    def tile_shape(self, idx: SomeIndex2d) -> Shape2d:
+        """
+        Query shape for a given tile.
+
+        :param idx: ``(row, col)`` chunk index
+        :returns: shape of a tile
+        :raises: :py:class:`IndexError` when index is outside of ``[(0,0) -> .shape)``.
+        """
+        idx = iyx_(idx)
+        ny, nx = (int(a[i + 1]) - int(a[i]) for a, i in zip(self._offsets, idx.yx))
+        return Shape2d(x=nx, y=ny)
+
+    @property
+    def shape(self) -> Shape2d:
+        """Number of tiles along each dimension."""
+        ny, nx = (len(idx) - 1 for idx in self._offsets)
+        return Shape2d(x=nx, y=ny)
+
+    @property
+    def base(self) -> Shape2d:
+        """Base shape."""
+        ny, nx = (int(idx[-1]) for idx in self._offsets)
+        return Shape2d(x=nx, y=ny)
+
+    @property
+    def chunks(self) -> Tuple[Tuple[int, ...], Tuple[int, ...]]:
+        """Dask compatible chunk rerpesentation."""
+        y, x = (tuple(np.diff(idx).tolist()) for idx in self._offsets)
+        return (y, x)
+
+    def locate(self, pix: SomeIndex2d) -> Tuple[int, int]:
+        """Tile index from pixel coordinate."""
+        NY, NX = self.base.yx
+        y, x = iyx_(pix).yx
+        if y < 0 or y >= NY or x < 0 or x >= NX:
+            raise IndexError()
+        y, x = (
+            # 1: because offsets start from 0, n1, n1+n2, ...
+            int(np.searchsorted(bins[1:], pix))
+            for bins, pix in zip(self._offsets, (y, x))
+        )
+        return (y, x)
+
+    def __dask_tokenize__(self):
+        return (
+            "odc.geo.roi.VariableSizedTiles",
+            *self._offsets,
+        )
+
+    def __str__(self) -> str:
+        b1, b2, b3 = map(_fmt_shape, [self.shape, self.tile_shape((0, 0)), self.base])
+        return f"Tiles: {b1}|chunked {b2}px => {b3}px"
+
+    def __repr__(self) -> str:
+        return self.__str__()
+
+    def __eq__(self, __value: object) -> bool:
+        if __value is self:
+            return True
+        if not isinstance(__value, VariableSizedTiles):
+            return False
+
+        for a, b in zip(self._offsets, __value._offsets):
+            if a.shape != b.shape:
+                return False
+            if (a != b).any():
+                return False
+        return True
+
+
+def roi_tiles(shape: SomeShape, how: Union[SomeShape, Chunks2d]) -> RoiTiles:
+    if isinstance(how, (tuple, list)) and isinstance(how[0], (tuple, list)):
+        y, x = (tuple(i) for i in how)
+        return VariableSizedTiles((y, x))
+    return Tiles(shape, how)
+
 
 def polygon_path(
     x: np.ndarray,
@@ -226,6 +449,14 @@ def scaled_down_shape(shape: Tuple[int, ...], scale: int) -> Tuple[int, ...]:
     return tuple(align_up(s, scale) // scale for s in shape)
 
 
+# fmt: off
+@overload
+def roi_shape(roi: ROI) -> Tuple[int, int]: ...
+@overload
+def roi_shape(roi: NdROI) -> Tuple[int, ...]: ...
+# fmt: on
+
+
 def roi_shape(roi: NdROI) -> Tuple[int, ...]:
     """
     Shape of an array after cropping with ``roi``.
@@ -308,6 +539,8 @@ def _norm_slice_or_error(s: SomeSlice) -> NormalizedSlice:
 
 def _norm_slice(s: SomeSlice, n: int) -> NormalizedSlice:
     if isinstance(s, int):
+        if s < 0:
+            s = n + s
         return slice(s, s + 1)
     start = _fill_if_none(s.start, 0)
     stop = _fill_if_none(s.stop, n)
@@ -315,10 +548,57 @@ def _norm_slice(s: SomeSlice, n: int) -> NormalizedSlice:
     return slice(start, stop, s.step)
 
 
+def slice_intersect3(a: SomeSlice, b: SomeSlice) -> Tuple[slice, slice, slice]:
+    """
+    Compute overlap 3 way.
+
+    Compute part of a that overlaps b, part of b that overlaps a and the region
+    of the original region covered by the overlap.
+
+    :returns: ``a', b', ab'``, such that ``X[a][a'] == X[b][b'] == X[ab']``
+    """
+    a = _norm_slice_or_error(a)
+    b = _norm_slice_or_error(b)
+    na = a.stop - a.start
+    nb = b.stop - b.start
+
+    if a.stop < b.start:
+        return slice(na, na), slice(0, 0), slice(a.stop, a.stop)
+    if a.start > b.stop:
+        return slice(0, 0), slice(nb, nb), slice(a.start, a.start)
+
+    _in = max(a.start, b.start)
+    _out = min(a.stop, b.stop)
+
+    return (
+        slice(_in - a.start, _out - a.start),
+        slice(_in - b.start, _out - b.start),
+        slice(_in, _out),
+    )
+
+
+def roi_intersect3(
+    a: Tuple[SomeSlice, ...], b: Tuple[SomeSlice, ...]
+) -> Tuple[Tuple[slice, ...], Tuple[slice, ...], Tuple[slice, ...]]:
+    """
+    Compute overlap 3 way.
+
+    Compute part of a that overlaps b, part of b that overlaps a and the region
+    of the original region covered by the overlap.
+
+    :returns: ``a', b', ab'``, such that ``X[a][a'] == X[b][b'] == X[ab']``
+    """
+    assert len(a) == len(b)
+    aa, bb, cc = zip(*[slice_intersect3(a_, b_) for a_, b_ in zip(a, b)])
+    return aa, bb, cc
+
+
 # fmt: off
 @overload
 def roi_normalise(roi: SomeSlice, shape: Union[int, Tuple[int]]) -> NormalizedSlice: ...
 @overload
+def roi_normalise(roi: ROI, shape: Tuple[int, int]) -> NormalizedROI: ...
+@overload
 def roi_normalise(roi: Tuple[SomeSlice, ...], shape: Tuple[int, ...]
 ) -> Tuple[NormalizedSlice, ...]: ...
 # fmt: on
